package be.smals.ehealth.etee.domain.identifier;

import org.apache.commons.lang.Validate;

/**
 * Utility class that provides implementations of {@link CheckDigitAlgorithm}.
 *
 * @author matla
 * @since 20141.0.0
 */
public final class CheckDigitAlgorithmFactory {

    private static final CheckDigitAlgorithm CHECK_MODULO97 = new CheckDigitAlgorithm() {
        @Override
        public boolean validate(String identifier) {
            long checksum = (asLong(identifier) / 100) % 97;
            checksum = (checksum != 0) ? checksum : 97;
            return checksum == asLong(identifier) % 100;
        }
    };

    private static final CheckDigitAlgorithm CHECK_97MODULO97 = new CheckDigitAlgorithm() {
        @Override
        public boolean validate(String identifier) {
            long checksum = 97 - (asLong(identifier) / 100 % 97);
            checksum = (checksum != 0) ? checksum : 97;
            return checksum == asLong(identifier) % 100;
        }
    };

    private static final CheckDigitAlgorithm CHECK_89MODULO89 = new CheckDigitAlgorithm() {
        @Override
        public boolean validate(String identifier) {
            long checksum = 89 - (asLong(identifier) / 100 % 89);
            checksum = (checksum != 0) ? checksum : 89;
            return checksum == asLong(identifier) % 100;
        }
    };

    public static CheckDigitAlgorithm create(String algorithm) {
        Validate.notEmpty(algorithm);
        if (algorithm.equals("97MOD97")) {
            return CHECK_97MODULO97;
        } else if (algorithm.equals("89MOD89")) {
            return CHECK_89MODULO89;
        }
        return CHECK_MODULO97;
    }

    public static CheckDigitAlgorithm createModulo97() {
        return CHECK_MODULO97;
    }

    public static CheckDigitAlgorithm create97Modulo97() {
        return CHECK_97MODULO97;
    }

    public static CheckDigitAlgorithm create89Modulo89() {
        return CHECK_89MODULO89;
    }

    /**
     * Returns the given identifier as a long. If the identifier is not a valid Long value, 0 will be returned. This
     * shouldn't cause any problems because the algorithm will fail anyway.
     *
     * @return The identifier as a long value.
     */
    private static long asLong(String identifier) {
        try {
            return Long.parseLong(identifier);
        } catch (NumberFormatException e) {
            return 0L;
        }
    }


}